@@ -1,34 +1,21 @@
-/***************************************************************************
- *   Copyright (C) 2015 Kai Uwe Broulik <kde@privat.broulik.de>            *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2015 Kai Uwe Broulik <kde@privat.broulik.de>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include "kwinkscreenhelpereffect.h"
 
 #include <QCoreApplication>
-#include <QX11Info>
 #include <chrono>
+#include <private/qtx11extras_p.h>
 
 using namespace std::chrono_literals;
 
 namespace PowerDevil
 {
-
-KWinKScreenHelperEffect::KWinKScreenHelperEffect(QObject *parent) : QObject(parent)
+KWinKScreenHelperEffect::KWinKScreenHelperEffect(QObject *parent)
+    : QObject(parent)
 {
     m_abortTimer.setSingleShot(true);
     m_abortTimer.setInterval(11s);
@@ -71,14 +58,12 @@ void KWinKScreenHelperEffect::stop()
 
 bool KWinKScreenHelperEffect::checkValid()
 {
-#ifdef HAVE_XCB
+#if HAVE_XCB
     if (QX11Info::isPlatformX11()) {
-        QScopedPointer<xcb_list_properties_reply_t, QScopedPointerPodDeleter> propsReply(xcb_list_properties_reply(QX11Info::connection(),
-            xcb_list_properties_unchecked(QX11Info::connection(), QX11Info::appRootWindow()),
-        nullptr));
-        QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atomReply(xcb_intern_atom_reply(QX11Info::connection(),
-            xcb_intern_atom_unchecked(QX11Info::connection(), false, 25, "_KDE_KWIN_KSCREEN_SUPPORT"),
-        nullptr));
+        QScopedPointer<xcb_list_properties_reply_t, QScopedPointerPodDeleter> propsReply(
+            xcb_list_properties_reply(QX11Info::connection(), xcb_list_properties_unchecked(QX11Info::connection(), QX11Info::appRootWindow()), nullptr));
+        QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atomReply(
+            xcb_intern_atom_reply(QX11Info::connection(), xcb_intern_atom_unchecked(QX11Info::connection(), false, 25, "_KDE_KWIN_KSCREEN_SUPPORT"), nullptr));
 
         if (propsReply.isNull() || atomReply.isNull()) {
             return false;
@@ -101,7 +86,7 @@ bool KWinKScreenHelperEffect::checkValid()
 
 void KWinKScreenHelperEffect::setEffectProperty(long value)
 {
-#ifdef HAVE_XCB
+#if HAVE_XCB
     if (m_isValid && QX11Info::isPlatformX11()) {
         xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, QX11Info::appRootWindow(), m_atom, XCB_ATOM_CARDINAL, 32, 1, &value);
     }
@@ -110,7 +95,7 @@ void KWinKScreenHelperEffect::setEffectProperty(long value)
 #endif
 }
 
-bool KWinKScreenHelperEffect::nativeEventFilter(const QByteArray &eventType, void *message, long int *result)
+bool KWinKScreenHelperEffect::nativeEventFilter(const QByteArray &eventType, void *message, qintptr *result)
 {
     Q_UNUSED(result);
 
@@ -118,7 +103,7 @@ bool KWinKScreenHelperEffect::nativeEventFilter(const QByteArray &eventType, voi
         return false;
     }
 
-#ifdef HAVE_XCB
+#if HAVE_XCB
     if (m_isValid && QX11Info::isPlatformX11()) {
         auto e = static_cast<xcb_generic_event_t *>(message);
         const uint8_t type = e->response_type & ~0x80;
@@ -140,7 +125,7 @@ bool KWinKScreenHelperEffect::nativeEventFilter(const QByteArray &eventType, voi
                     return false;
                 }
 
-                switch(*data) {
+                switch (*data) {
                 case 1:
                     m_state = FadingOutState;
                     break;
@@ -172,3 +157,5 @@ bool KWinKScreenHelperEffect::nativeEventFilter(const QByteArray &eventType, voi
 }
 
 } // namespace PowerDevil
+
+#include "moc_kwinkscreenhelpereffect.cpp"
