@@ -2,6 +2,7 @@
  *   SPDX-FileCopyrightText: 2010 Dario Freddi <drf@kde.org>
  *   SPDX-FileCopyrightText: 2012 Lukáš Tinkl <ltinkl@redhat.com>
  *   SPDX-FileCopyrightText: 2016 Kai Uwe Broulik <kde@privat.broulik.de>
+ *   SPDX-FileCopyrightText: 2024 Natalie Clarius <natalie.clarius@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -16,18 +17,25 @@
 #include <QDBusPendingReply>
 #include <QDBusServiceWatcher>
 #include <QMetaType>
+#include <QPair>
 #include <QTimer>
+#include <QVariant>
 
+#include <KConfigGroup>
 #include <KIdleTime>
+#include <KSharedConfig>
 
 #include <algorithm>
 #include <unistd.h>
 
+#include "PowerDevilGlobalSettings.h"
 #include "powerdevil_debug.h"
 #include "powerdevilpolicyagent.h"
 
 #include "screenlocker_interface.h"
 
+using namespace Qt::StringLiterals;
+
 struct NamedDBusObjectPath {
     QString name;
     QDBusObjectPath path;
@@ -123,7 +131,7 @@ PolicyAgent::~PolicyAgent()
 {
 }
 
-void PolicyAgent::init()
+void PolicyAgent::init(GlobalSettings *globalSettings)
 {
     qDBusRegisterMetaType<InhibitionInfo>();
     qDBusRegisterMetaType<QList<InhibitionInfo>>();
@@ -178,6 +186,18 @@ void PolicyAgent::init()
         }
         watcher->deleteLater();
     });
+
+    m_config = globalSettings;
+    // get comma separated list of colon separated app - reason inhibition pairs
+    // that have been configured to be blocked
+    QStringList blockedInhibitions = m_config->blockedInhibitions();
+    std::ranges::transform(std::as_const(blockedInhibitions),
+                           std::inserter(m_configuredToBlockInhibitions, m_configuredToBlockInhibitions.begin()),
+                           [](const QString &inhibition) {
+                               const QString app = inhibition.section(QLatin1Char(':'), 0, 0);
+                               const QString reason = inhibition.section(QLatin1Char(':'), 1);
+                               return qMakePair(app, reason);
+                           });
 }
 
 QString PolicyAgent::getNamedPathProperty(const QString &path, const QString &iface, const QString &prop) const
@@ -237,9 +257,9 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
         }
 
         // now let's obtain the seat
-        QString seatPath = getNamedPathProperty(sessionPath, SYSTEMD_LOGIN1_SESSION_IFACE, "Seat");
+        QString seatPath = getNamedPathProperty(sessionPath, SYSTEMD_LOGIN1_SESSION_IFACE, u"Seat"_s);
 
-        if (seatPath.isEmpty() || seatPath == "/") {
+        if (seatPath.isEmpty() || seatPath == u'/') {
             qCDebug(POWERDEVIL) << "Unable to associate systemd session with a seat" << seatPath;
             m_sdAvailable = false;
             return;
@@ -257,13 +277,13 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
         }
 
         // finally get the active session path and watch for its changes
-        m_activeSessionPath = getNamedPathProperty(seatPath, SYSTEMD_LOGIN1_SEAT_IFACE, "ActiveSession");
+        m_activeSessionPath = getNamedPathProperty(seatPath, SYSTEMD_LOGIN1_SEAT_IFACE, u"ActiveSession"_s);
 
         qCDebug(POWERDEVIL) << "ACTIVE SESSION PATH:" << m_activeSessionPath;
         QDBusConnection::systemBus().connect(SYSTEMD_LOGIN1_SERVICE,
                                              seatPath,
-                                             "org.freedesktop.DBus.Properties",
-                                             "PropertiesChanged",
+                                             u"org.freedesktop.DBus.Properties"_s,
+                                             u"PropertiesChanged"_s,
                                              this,
                                              SLOT(onActiveSessionChanged(QString, QVariantMap, QStringList)));
 
@@ -274,8 +294,8 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
         // and then track logind's ihibitions, too
         QDBusConnection::systemBus().connect(SYSTEMD_LOGIN1_SERVICE,
                                              SYSTEMD_LOGIN1_PATH,
-                                             "org.freedesktop.DBus.Properties",
-                                             "PropertiesChanged",
+                                             u"org.freedesktop.DBus.Properties"_s,
+                                             u"PropertiesChanged"_s,
                                              this,
                                              SLOT(onManagerPropertyChanged(QString, QVariantMap, QStringList)));
         checkLogindInhibitions();
@@ -293,7 +313,7 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
             return;
         }
 
-        QDBusPendingReply<QDBusObjectPath> sessionPath = m_managerIface.data()->asyncCall("GetCurrentSession");
+        QDBusPendingReply<QDBusObjectPath> sessionPath = m_managerIface.data()->asyncCall(u"GetCurrentSession"_s);
 
         sessionPath.waitForFinished();
 
@@ -304,7 +324,7 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
         }
 
         m_ckSessionInterface =
-            new QDBusInterface(CONSOLEKIT_SERVICE, sessionPath.value().path(), "org.freedesktop.ConsoleKit.Session", QDBusConnection::systemBus());
+            new QDBusInterface(CONSOLEKIT_SERVICE, sessionPath.value().path(), u"org.freedesktop.ConsoleKit.Session"_s, QDBusConnection::systemBus());
 
         if (!m_ckSessionInterface.data()->isValid()) {
             // As above
@@ -325,8 +345,8 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
 
         if (!QDBusConnection::systemBus().connect(CONSOLEKIT_SERVICE,
                                                   seatPath.value().path(),
-                                                  "org.freedesktop.ConsoleKit.Seat",
-                                                  "ActiveSessionChanged",
+                                                  u"org.freedesktop.ConsoleKit.Seat"_s,
+                                                  u"ActiveSessionChanged"_s,
                                                   this,
                                                   SLOT(onActiveSessionChanged(QString)))) {
             qCDebug(POWERDEVIL) << "Unable to connect to ActiveSessionChanged";
@@ -335,7 +355,8 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
         }
 
         // Force triggering of active session changed
-        QDBusMessage call = QDBusMessage::createMethodCall(CONSOLEKIT_SERVICE, seatPath.value().path(), "org.freedesktop.ConsoleKit.Seat", "GetActiveSession");
+        QDBusMessage call =
+            QDBusMessage::createMethodCall(CONSOLEKIT_SERVICE, seatPath.value().path(), u"org.freedesktop.ConsoleKit.Seat"_s, u"GetActiveSession"_s);
         QDBusPendingReply<QDBusObjectPath> activeSession = QDBusConnection::systemBus().asyncCall(call);
         activeSession.waitForFinished();
 
@@ -350,7 +371,7 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
 
 void PolicyAgent::onSessionHandlerUnregistered(const QString &serviceName)
 {
-    if (serviceName == QLatin1String(SYSTEMD_LOGIN1_SERVICE)) {
+    if (serviceName == SYSTEMD_LOGIN1_SERVICE) {
         m_sdAvailable = false;
         delete m_sdSessionInterface.data();
     } else if (serviceName == QLatin1String(CONSOLEKIT_SERVICE)) {
@@ -587,6 +608,8 @@ void PolicyAgent::onScreenLockerActiveChanged(bool active)
 
 uint PolicyAgent::addInhibitionWithExplicitDBusService(uint types, const QString &appName, const QString &reason, const QString &service)
 {
+    InhibitionInfo info = qMakePair(appName, reason);
+
     ++m_lastCookie;
 
     const int cookie = m_lastCookie; // when the Timer below fires, m_lastCookie might be different already
@@ -611,11 +634,86 @@ uint PolicyAgent::addInhibitionWithExplicitDBusService(uint types, const QString
             return;
         }
 
-        m_cookieToAppName.insert(cookie, qMakePair(appName, reason));
+        InhibitionInfo info = qMakePair(appName, reason);
+        m_cookieToAppName.insert(cookie, info);
+
+        // when inhibition gets blocked later, revoke it if it is still active
+        m_blockInhibitionConnections.insert(cookie,
+                                            connect(this,
+                                                    &PolicyAgent::blockInhibitionRequested,
+                                                    this,
+                                                    [this, cookie, service, appName, reason](const InhibitionInfo &blockedInfo, const bool permanently) {
+                                                        InhibitionInfo info = qMakePair(appName, reason);
+                                                        if (blockedInfo == info && m_activeInhibitions.contains(cookie)) {
+                                                            qCDebug(POWERDEVIL)
+                                                                << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
+                                                                << "was configured by the user to be blocked; releasing";
+
+                                                            ReleaseInhibition(cookie, true);
+
+                                                            if (m_blockedInhibitions.contains(cookie)) {
+                                                                // was already blocked but differently (temporarily vs permanently)
+                                                                m_blockedInhibitions.remove(cookie);
+                                                                if (m_configuredToBlockInhibitions.contains(info)) {
+                                                                    Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+                                                                } else {
+                                                                    Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+                                                                }
+                                                            }
+                                                            m_blockedInhibitions.insert(cookie);
+                                                            if (permanently) {
+                                                                if (!m_configuredToBlockInhibitions.contains(info)) {
+                                                                    // newly permaanently blocked, ie not reblocked after temporarily unblocked
+                                                                    Q_EMIT PermanentlyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
+                                                                }
+                                                            } else {
+                                                                Q_EMIT TemporarilyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
+                                                            }
+                                                        }
+                                                    }));
+
+        // when inhibition gets unblocked later, retroactively add it if it still exists
+        m_unblockInhibitionConnections.insert(
+            cookie,
+            connect(this,
+                    &PolicyAgent::unblockInhibitionRequested,
+                    this,
+                    [this, cookie, service, appName, reason, types](const InhibitionInfo &unblockedInfo, const bool permanently) {
+                        InhibitionInfo info = qMakePair(appName, reason);
+                        if (unblockedInfo == info && m_blockedInhibitions.contains(cookie) && m_cookieToAppName.contains(cookie)) {
+                            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
+                                                << "was configured by the user to be unblocked; adding retroactively";
+
+                            addInhibitionTypeHelper(cookie, static_cast<PolicyAgent::RequiredPolicies>(types));
+                            m_activeInhibitions.insert(cookie);
+                            Q_EMIT InhibitionsChanged({{info}}, {});
+                            m_pendingInhibitions.removeOne(cookie);
+
+                            m_blockedInhibitions.remove(cookie);
+                            if (permanently) {
+                                Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+                            } else {
+                                Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+                            }
+                        }
+                    }));
+
+        // reject inhibitions configured as blocked
+        if (m_configuredToBlockInhibitions.contains(info)) {
+            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
+                                << "has been configured by the user to be blocked; rejecting";
+
+            m_pendingInhibitions.removeOne(cookie);
+            m_blockedInhibitions.insert(cookie);
+            Q_EMIT PermanentlyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
+            return;
+        }
 
         addInhibitionTypeHelper(cookie, static_cast<PolicyAgent::RequiredPolicies>(types));
 
-        Q_EMIT InhibitionsChanged({{qMakePair(appName, reason)}}, {});
+        m_activeInhibitions.insert(cookie);
+
+        Q_EMIT InhibitionsChanged({{info}}, {});
 
         m_pendingInhibitions.removeOne(cookie);
     });
@@ -660,7 +758,7 @@ void PolicyAgent::addInhibitionTypeHelper(uint cookie, PolicyAgent::RequiredPoli
     }
 }
 
-void PolicyAgent::ReleaseInhibition(uint cookie)
+void PolicyAgent::ReleaseInhibition(uint cookie, bool retainCookie)
 {
     qCDebug(POWERDEVIL) << "Releasing inhibition with cookie " << cookie;
 
@@ -675,8 +773,34 @@ void PolicyAgent::ReleaseInhibition(uint cookie)
         return;
     }
 
+    m_activeInhibitions.remove(cookie);
     Q_EMIT InhibitionsChanged(QList<InhibitionInfo>(), {{m_cookieToAppName.value(cookie).first}});
-    m_cookieToAppName.remove(cookie);
+
+    if (m_blockedInhibitions.remove(cookie)) {
+        qCDebug(POWERDEVIL) << "It was blocked, just discarding it";
+        InhibitionInfo info = m_cookieToAppName.take(cookie);
+        m_cookieToAppName.remove(cookie);
+        if (m_configuredToBlockInhibitions.contains(info)) {
+            Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+        } else {
+            Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
+        }
+        return;
+    }
+
+    // Delete information about the inhibition,
+    // but not when it is being released because it has been blocked,
+    // we may need to restore it later
+    if (!retainCookie) {
+        m_cookieToAppName.remove(cookie);
+
+        if (m_blockInhibitionConnections.contains(cookie)) {
+            disconnect(m_blockInhibitionConnections.take(cookie));
+        }
+        if (m_unblockInhibitionConnections.contains(cookie)) {
+            disconnect(m_unblockInhibitionConnections.take(cookie));
+        }
+    }
 
     // Look through all of the inhibition types
     bool notify = false;
@@ -703,7 +827,9 @@ void PolicyAgent::ReleaseInhibition(uint cookie)
 
 QList<InhibitionInfo> PolicyAgent::ListInhibitions() const
 {
-    return m_cookieToAppName.values();
+    return std::transform_reduce(m_activeInhibitions.constBegin(), m_activeInhibitions.constEnd(), QList<InhibitionInfo>(), std::plus<>(), [this](uint cookie) {
+        return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
+    });
 }
 
 bool PolicyAgent::HasInhibition(/*PolicyAgent::RequiredPolicies*/ uint types)
@@ -747,6 +873,87 @@ void PolicyAgent::setupSystemdInhibition()
         qCWarning(POWERDEVIL) << "failed to inhibit systemd powersave handling";
 }
 
+void PolicyAgent::BlockInhibition(const QString &appName, const QString &reason, bool permanently)
+{
+    InhibitionInfo info = qMakePair(appName, reason);
+    qCDebug(POWERDEVIL) << "Blocking inhibitions from" << appName << "with reason" << reason;
+    Q_EMIT blockInhibitionRequested(info, permanently);
+    if (!permanently) {
+        return;
+    }
+
+    if (m_configuredToBlockInhibitions.contains(qMakePair(appName, reason))) {
+        qCDebug(POWERDEVIL) << "Inhibitions from" << appName << "with reason" << reason << "are already blocked";
+        return;
+    }
+    qCDebug(POWERDEVIL) << "Adding inhibitions from" << appName << "with reason" << reason << "to list of permanently blocked inhibitions";
+
+    m_configuredToBlockInhibitions.insert(qMakePair(appName, reason));
+    QStringList configuredBlockedInhibitions;
+    std::ranges::transform(std::as_const(m_configuredToBlockInhibitions),
+                           std::back_inserter(configuredBlockedInhibitions),
+                           [](const QPair<QString, QString> &inhibition) {
+                               return inhibition.first + QLatin1Char(':') + inhibition.second;
+                           });
+    m_config->setBlockedInhibitions(configuredBlockedInhibitions);
+    m_config->save();
 }
 
+void PolicyAgent::UnblockInhibition(const QString &appName, const QString &reason, bool permanently)
+{
+    InhibitionInfo info = qMakePair(appName, reason);
+    qCDebug(POWERDEVIL) << "Unblocking inhibitions from" << appName << "with reason" << reason;
+    Q_EMIT unblockInhibitionRequested(info, permanently);
+    if (!permanently) {
+        return;
+    }
+
+    if (!m_configuredToBlockInhibitions.contains(qMakePair(appName, reason))) {
+        qCDebug(POWERDEVIL) << "Inhibitions from" << appName << "with reason" << reason << "are not blocked";
+        return;
+    }
+    qCDebug(POWERDEVIL) << "Removing inhibitions from" << appName << "with reason" << reason << "from list of permanently blocked inhibitions";
+
+    m_configuredToBlockInhibitions.remove(qMakePair(appName, reason));
+    QStringList configuredBlockedInhibitions;
+    std::ranges::transform(std::as_const(m_configuredToBlockInhibitions),
+                           std::back_inserter(configuredBlockedInhibitions),
+                           [](const QPair<QString, QString> &inhibition) {
+                               return inhibition.first + QLatin1Char(':') + inhibition.second;
+                           });
+    m_config->setBlockedInhibitions(configuredBlockedInhibitions);
+    m_config->save();
+}
+
+QList<InhibitionInfo> PolicyAgent::ListPermanentlyBlockedInhibitions() const
+{
+    QList<uint> permanentlyBlockedInhibitions;
+    std::copy_if(m_blockedInhibitions.begin(), m_blockedInhibitions.end(), std::back_inserter(permanentlyBlockedInhibitions), [this](uint cookie) {
+        return m_configuredToBlockInhibitions.contains(m_cookieToAppName.value(cookie));
+    });
+    return std::transform_reduce(permanentlyBlockedInhibitions.constBegin(),
+                                 permanentlyBlockedInhibitions.constEnd(),
+                                 QList<InhibitionInfo>(),
+                                 std::plus<>(),
+                                 [this](uint cookie) {
+                                     return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
+                                 });
+}
+
+QList<InhibitionInfo> PolicyAgent::ListTemporarilyBlockedInhibitions() const
+{
+    QList<uint> temporarilyBlockedInhibitions;
+    std::copy_if(m_blockedInhibitions.begin(), m_blockedInhibitions.end(), std::back_inserter(temporarilyBlockedInhibitions), [this](uint cookie) {
+        return !m_configuredToBlockInhibitions.contains(m_cookieToAppName.value(cookie));
+    });
+    return std::transform_reduce(temporarilyBlockedInhibitions.constBegin(),
+                                 temporarilyBlockedInhibitions.constEnd(),
+                                 QList<InhibitionInfo>(),
+                                 std::plus<>(),
+                                 [this](uint cookie) {
+                                     return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
+                                 });
+}
+} // namespace PowerDevil
+
 #include "moc_powerdevilpolicyagent.cpp"
